package com.example.orderingsoftware.test;

import static org.junit.Assert.assertNotNull;

import com.example.orderingsoftware.PersistenceManager;

public abstract class AbstractPersistenceManagerTest {
	protected void runInPersistenceManager(IPersistenceManagerRunnable run) {
		assertNotNull(run);
		try (PersistenceManager manager = new PersistenceManager()) {
			try {
				manager.beginTransaction();
				try {
					run.run(manager);
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}
			finally {
				manager.endTransaction();
			}
		}
	}
	
	protected interface IPersistenceManagerRunnable {
		public void run(PersistenceManager manager) throws Exception;
	}
	
	protected <T> T runInPersistenceManagerWithResult(IPersistenceManagerRunnableWithResult<T> run) {
		assertNotNull(run);
		T result;
		try (PersistenceManager manager = new PersistenceManager()) {
			try {
				manager.beginTransaction();
				try {
					result = run.run(manager);
				} catch (Exception e) {
					throw new RuntimeException(e);
				}
			}
			finally {
				manager.endTransaction();
			}
		}
		return result;
	}
	
	protected interface IPersistenceManagerRunnableWithResult<T> {
		public T run(PersistenceManager manager) throws Exception;
	}
}