package com.yakindu.ea.examples.orderingsoftware.template.jpa

import com.yakindu.ea.examples.orderingsoftware.template.core.ClassTemplate
import com.yakindu.bridges.ea.examples.runtime.codegen.EACodegen
import com.yakindu.bridges.ea.examples.runtime.codegen.EACodegenFile
import java.util.Set
import org.eclipse.core.resources.IFile
import org.eclipse.uml2.uml.Class
import org.eclipse.uml2.uml.Property

class JPAClassTemplate extends ClassTemplate {
	@EACodegenFile
	def IFile path(Class element, IFile defaultFile) {
		return getTargetFilePath("jpa", "com/example/orderingsoftware/" + defaultFile.name)
	}
	
	@EACodegen("java")
	override generate(Class element) {
		return super.generate(element)
	}
	
	override printClassDeclaration(Class element) '''
		«IF element.isAbstract»
			@MappedSuperclass
		«ELSE»
			@Entity
			@Table(name = "«element.name»Table")
		«ENDIF»
		«super.printClassDeclaration(element)»'''
	
	override printAttribute(Property attribute) '''
		«IF attribute.ID»
			@Id
		«ENDIF»
		«IF attribute.type instanceof Class»
			«IF 1 == attribute.upper»
				@ManyToOne
			«ELSE»
				@OneToMany
			«ENDIF»
		«ENDIF»
		«super.printAttribute(attribute)»
	'''
	
	override makeFinal(Property attribute) {
		return false
	}
	
	override Set<String> collectImports(Class element) {
		val imports = super.collectImports(element)
		if (element.abstract) {
			imports.add("javax.persistence.MappedSuperclass")
		}
		else {
			imports.add("javax.persistence.Entity")
			imports.add("javax.persistence.Table")
		}
		element.ownedAttributes.forEach[attribute |
			if (attribute.type instanceof Class) {
				if (1 == attribute.upper) {
					imports.add("javax.persistence.ManyToOne")
				}
				else {
					imports.add("javax.persistence.OneToMany")
				}
			}
		]
		return imports
	}
	
	override collectImports(Property attribute, Set<String> datatypes) {
		super.collectImports(attribute, datatypes)
		if (attribute.ID) {
			datatypes.add("javax.persistence.Id")
		}
	}
}